<?php

namespace SuperRhino\Utils\Common;

use ReflectionObject;

class Util {

    /**
     * Class casting
     *
     * @param string|object $destination
     * @param object $sourceObject
     * @return object
     */
    function cast($destination, $sourceObject) {
        if (is_string($destination)) {
            $destination = new $destination();
        }
        $sourceReflection = new ReflectionObject($sourceObject);
        $destinationReflection = new ReflectionObject($destination);
        $sourceProperties = $sourceReflection->getProperties();
        foreach ($sourceProperties as $sourceProperty) {
            $sourceProperty->setAccessible(true);
            $name = $sourceProperty->getName();
            $value = $sourceProperty->getValue($sourceObject);
            if ($destinationReflection->hasProperty($name)) {
                $propDest = $destinationReflection->getProperty($name);
                $propDest->setAccessible(true);
                $propDest->setValue($destination, $value);
            } else {
                $destination->$name = $value;
            }
        }
        return $destination;
    }

    /**
     * 对月进行加减
     * @param int $date 时间戳
     * @param string $algorithm 比如 +1 month 或者-1 month
     * @return string
     */
    public static function calculate_month(int $date, string $algorithm): string {
        $day = date('d', $date);
        $newDateFirst = date('Y-m', strtotime('first day of ' . $algorithm, $date));
        $newDate = $newDateFirst . '-' . $day;
        $newDateLast = date('Y-m-d', strtotime('last day of ' . $algorithm, $date));
        return $newDateLast < $newDate ? $newDateLast : $newDate;
    }
}